/*******************************************************************************
* File Name: LampPWM.c
* Version 3.30
*
* Description:
*  The PWM User Module consist of an 8 or 16-bit counter with two 8 or 16-bit
*  comparitors. Each instance of this user module is capable of generating
*  two PWM outputs with the same period. The pulse width is selectable between
*  1 and 255/65535. The period is selectable between 2 and 255/65536 clocks.
*  The compare value output may be configured to be active when the present
*  counter is less than or less than/equal to the compare value.
*  A terminal count output is also provided. It generates a pulse one clock
*  width wide when the counter is equal to zero.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "LampPWM.h"

/* Error message for removed <resource> through optimization */
#ifdef LampPWM_PWMUDB_genblk1_ctrlreg__REMOVED
    #error PWM_v3_30 detected with a constant 0 for the enable or \
         constant 1 for reset. This will prevent the component from operating.
#endif /* LampPWM_PWMUDB_genblk1_ctrlreg__REMOVED */

uint8 LampPWM_initVar = 0u;


/*******************************************************************************
* Function Name: LampPWM_Start
********************************************************************************
*
* Summary:
*  The start function initializes the pwm with the default values, the
*  enables the counter to begin counting.  It does not enable interrupts,
*  the EnableInt command should be called if interrupt generation is required.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  LampPWM_initVar: Is modified when this function is called for the
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void LampPWM_Start(void) 
{
    /* If not Initialized then initialize all required hardware and software */
    if(LampPWM_initVar == 0u)
    {
        LampPWM_Init();
        LampPWM_initVar = 1u;
    }
    LampPWM_Enable();

}


/*******************************************************************************
* Function Name: LampPWM_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the
*  customizer of the component placed onto schematic. Usually called in
*  LampPWM_Start().
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void LampPWM_Init(void) 
{
    #if (LampPWM_UsingFixedFunction || LampPWM_UseControl)
        uint8 ctrl;
    #endif /* (LampPWM_UsingFixedFunction || LampPWM_UseControl) */

    #if(!LampPWM_UsingFixedFunction)
        #if(LampPWM_UseStatus)
            /* Interrupt State Backup for Critical Region*/
            uint8 LampPWM_interruptState;
        #endif /* (LampPWM_UseStatus) */
    #endif /* (!LampPWM_UsingFixedFunction) */

    #if (LampPWM_UsingFixedFunction)
        /* You are allowed to write the compare value (FF only) */
        LampPWM_CONTROL |= LampPWM_CFG0_MODE;
        #if (LampPWM_DeadBand2_4)
            LampPWM_CONTROL |= LampPWM_CFG0_DB;
        #endif /* (LampPWM_DeadBand2_4) */

        ctrl = LampPWM_CONTROL3 & ((uint8 )(~LampPWM_CTRL_CMPMODE1_MASK));
        LampPWM_CONTROL3 = ctrl | LampPWM_DEFAULT_COMPARE1_MODE;

         /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
        LampPWM_RT1 &= ((uint8)(~LampPWM_RT1_MASK));
        LampPWM_RT1 |= LampPWM_SYNC;

        /*Enable DSI Sync all all inputs of the PWM*/
        LampPWM_RT1 &= ((uint8)(~LampPWM_SYNCDSI_MASK));
        LampPWM_RT1 |= LampPWM_SYNCDSI_EN;

    #elif (LampPWM_UseControl)
        /* Set the default compare mode defined in the parameter */
        ctrl = LampPWM_CONTROL & ((uint8)(~LampPWM_CTRL_CMPMODE2_MASK)) &
                ((uint8)(~LampPWM_CTRL_CMPMODE1_MASK));
        LampPWM_CONTROL = ctrl | LampPWM_DEFAULT_COMPARE2_MODE |
                                   LampPWM_DEFAULT_COMPARE1_MODE;
    #endif /* (LampPWM_UsingFixedFunction) */

    #if (!LampPWM_UsingFixedFunction)
        #if (LampPWM_Resolution == 8)
            /* Set FIFO 0 to 1 byte register for period*/
            LampPWM_AUX_CONTROLDP0 |= (LampPWM_AUX_CTRL_FIFO0_CLR);
        #else /* (LampPWM_Resolution == 16)*/
            /* Set FIFO 0 to 1 byte register for period */
            LampPWM_AUX_CONTROLDP0 |= (LampPWM_AUX_CTRL_FIFO0_CLR);
            LampPWM_AUX_CONTROLDP1 |= (LampPWM_AUX_CTRL_FIFO0_CLR);
        #endif /* (LampPWM_Resolution == 8) */

        LampPWM_WriteCounter(LampPWM_INIT_PERIOD_VALUE);
    #endif /* (!LampPWM_UsingFixedFunction) */

    LampPWM_WritePeriod(LampPWM_INIT_PERIOD_VALUE);

        #if (LampPWM_UseOneCompareMode)
            LampPWM_WriteCompare(LampPWM_INIT_COMPARE_VALUE1);
        #else
            LampPWM_WriteCompare1(LampPWM_INIT_COMPARE_VALUE1);
            LampPWM_WriteCompare2(LampPWM_INIT_COMPARE_VALUE2);
        #endif /* (LampPWM_UseOneCompareMode) */

        #if (LampPWM_KillModeMinTime)
            LampPWM_WriteKillTime(LampPWM_MinimumKillTime);
        #endif /* (LampPWM_KillModeMinTime) */

        #if (LampPWM_DeadBandUsed)
            LampPWM_WriteDeadTime(LampPWM_INIT_DEAD_TIME);
        #endif /* (LampPWM_DeadBandUsed) */

    #if (LampPWM_UseStatus || LampPWM_UsingFixedFunction)
        LampPWM_SetInterruptMode(LampPWM_INIT_INTERRUPTS_MODE);
    #endif /* (LampPWM_UseStatus || LampPWM_UsingFixedFunction) */

    #if (LampPWM_UsingFixedFunction)
        /* Globally Enable the Fixed Function Block chosen */
        LampPWM_GLOBAL_ENABLE |= LampPWM_BLOCK_EN_MASK;
        /* Set the Interrupt source to come from the status register */
        LampPWM_CONTROL2 |= LampPWM_CTRL2_IRQ_SEL;
    #else
        #if(LampPWM_UseStatus)

            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            LampPWM_interruptState = CyEnterCriticalSection();
            /* Use the interrupt output of the status register for IRQ output */
            LampPWM_STATUS_AUX_CTRL |= LampPWM_STATUS_ACTL_INT_EN_MASK;

             /* Exit Critical Region*/
            CyExitCriticalSection(LampPWM_interruptState);

            /* Clear the FIFO to enable the LampPWM_STATUS_FIFOFULL
                   bit to be set on FIFO full. */
            LampPWM_ClearFIFO();
        #endif /* (LampPWM_UseStatus) */
    #endif /* (LampPWM_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: LampPWM_Enable
********************************************************************************
*
* Summary:
*  Enables the PWM block operation
*
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  This works only if software enable mode is chosen
*
*******************************************************************************/
void LampPWM_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (LampPWM_UsingFixedFunction)
        LampPWM_GLOBAL_ENABLE |= LampPWM_BLOCK_EN_MASK;
        LampPWM_GLOBAL_STBY_ENABLE |= LampPWM_BLOCK_STBY_EN_MASK;
    #endif /* (LampPWM_UsingFixedFunction) */

    /* Enable the PWM from the control register  */
    #if (LampPWM_UseControl || LampPWM_UsingFixedFunction)
        LampPWM_CONTROL |= LampPWM_CTRL_ENABLE;
    #endif /* (LampPWM_UseControl || LampPWM_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: LampPWM_Stop
********************************************************************************
*
* Summary:
*  The stop function halts the PWM, but does not change any modes or disable
*  interrupts.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  If the Enable mode is set to Hardware only then this function
*  has no effect on the operation of the PWM
*
*******************************************************************************/
void LampPWM_Stop(void) 
{
    #if (LampPWM_UseControl || LampPWM_UsingFixedFunction)
        LampPWM_CONTROL &= ((uint8)(~LampPWM_CTRL_ENABLE));
    #endif /* (LampPWM_UseControl || LampPWM_UsingFixedFunction) */

    /* Globally disable the Fixed Function Block chosen */
    #if (LampPWM_UsingFixedFunction)
        LampPWM_GLOBAL_ENABLE &= ((uint8)(~LampPWM_BLOCK_EN_MASK));
        LampPWM_GLOBAL_STBY_ENABLE &= ((uint8)(~LampPWM_BLOCK_STBY_EN_MASK));
    #endif /* (LampPWM_UsingFixedFunction) */
}

#if (LampPWM_UseOneCompareMode)
    #if (LampPWM_CompareMode1SW)


        /*******************************************************************************
        * Function Name: LampPWM_SetCompareMode
        ********************************************************************************
        *
        * Summary:
        *  This function writes the Compare Mode for the pwm output when in Dither mode,
        *  Center Align Mode or One Output Mode.
        *
        * Parameters:
        *  comparemode:  The new compare mode for the PWM output. Use the compare types
        *                defined in the H file as input arguments.
        *
        * Return:
        *  None
        *
        *******************************************************************************/
        void LampPWM_SetCompareMode(uint8 comparemode) 
        {
            #if(LampPWM_UsingFixedFunction)

                #if(0 != LampPWM_CTRL_CMPMODE1_SHIFT)
                    uint8 comparemodemasked = ((uint8)((uint8)comparemode << LampPWM_CTRL_CMPMODE1_SHIFT));
                #else
                    uint8 comparemodemasked = comparemode;
                #endif /* (0 != LampPWM_CTRL_CMPMODE1_SHIFT) */

                LampPWM_CONTROL3 &= ((uint8)(~LampPWM_CTRL_CMPMODE1_MASK)); /*Clear Existing Data */
                LampPWM_CONTROL3 |= comparemodemasked;

            #elif (LampPWM_UseControl)

                #if(0 != LampPWM_CTRL_CMPMODE1_SHIFT)
                    uint8 comparemode1masked = ((uint8)((uint8)comparemode << LampPWM_CTRL_CMPMODE1_SHIFT)) &
                                                LampPWM_CTRL_CMPMODE1_MASK;
                #else
                    uint8 comparemode1masked = comparemode & LampPWM_CTRL_CMPMODE1_MASK;
                #endif /* (0 != LampPWM_CTRL_CMPMODE1_SHIFT) */

                #if(0 != LampPWM_CTRL_CMPMODE2_SHIFT)
                    uint8 comparemode2masked = ((uint8)((uint8)comparemode << LampPWM_CTRL_CMPMODE2_SHIFT)) &
                                               LampPWM_CTRL_CMPMODE2_MASK;
                #else
                    uint8 comparemode2masked = comparemode & LampPWM_CTRL_CMPMODE2_MASK;
                #endif /* (0 != LampPWM_CTRL_CMPMODE2_SHIFT) */

                /*Clear existing mode */
                LampPWM_CONTROL &= ((uint8)(~(LampPWM_CTRL_CMPMODE1_MASK |
                                            LampPWM_CTRL_CMPMODE2_MASK)));
                LampPWM_CONTROL |= (comparemode1masked | comparemode2masked);

            #else
                uint8 temp = comparemode;
            #endif /* (LampPWM_UsingFixedFunction) */
        }
    #endif /* LampPWM_CompareMode1SW */

#else /* UseOneCompareMode */

    #if (LampPWM_CompareMode1SW)


        /*******************************************************************************
        * Function Name: LampPWM_SetCompareMode1
        ********************************************************************************
        *
        * Summary:
        *  This function writes the Compare Mode for the pwm or pwm1 output
        *
        * Parameters:
        *  comparemode:  The new compare mode for the PWM output. Use the compare types
        *                defined in the H file as input arguments.
        *
        * Return:
        *  None
        *
        *******************************************************************************/
        void LampPWM_SetCompareMode1(uint8 comparemode) 
        {
            #if(0 != LampPWM_CTRL_CMPMODE1_SHIFT)
                uint8 comparemodemasked = ((uint8)((uint8)comparemode << LampPWM_CTRL_CMPMODE1_SHIFT)) &
                                           LampPWM_CTRL_CMPMODE1_MASK;
            #else
                uint8 comparemodemasked = comparemode & LampPWM_CTRL_CMPMODE1_MASK;
            #endif /* (0 != LampPWM_CTRL_CMPMODE1_SHIFT) */

            #if (LampPWM_UseControl)
                LampPWM_CONTROL &= ((uint8)(~LampPWM_CTRL_CMPMODE1_MASK)); /*Clear existing mode */
                LampPWM_CONTROL |= comparemodemasked;
            #endif /* (LampPWM_UseControl) */
        }
    #endif /* LampPWM_CompareMode1SW */

#if (LampPWM_CompareMode2SW)


    /*******************************************************************************
    * Function Name: LampPWM_SetCompareMode2
    ********************************************************************************
    *
    * Summary:
    *  This function writes the Compare Mode for the pwm or pwm2 output
    *
    * Parameters:
    *  comparemode:  The new compare mode for the PWM output. Use the compare types
    *                defined in the H file as input arguments.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void LampPWM_SetCompareMode2(uint8 comparemode) 
    {

        #if(0 != LampPWM_CTRL_CMPMODE2_SHIFT)
            uint8 comparemodemasked = ((uint8)((uint8)comparemode << LampPWM_CTRL_CMPMODE2_SHIFT)) &
                                                 LampPWM_CTRL_CMPMODE2_MASK;
        #else
            uint8 comparemodemasked = comparemode & LampPWM_CTRL_CMPMODE2_MASK;
        #endif /* (0 != LampPWM_CTRL_CMPMODE2_SHIFT) */

        #if (LampPWM_UseControl)
            LampPWM_CONTROL &= ((uint8)(~LampPWM_CTRL_CMPMODE2_MASK)); /*Clear existing mode */
            LampPWM_CONTROL |= comparemodemasked;
        #endif /* (LampPWM_UseControl) */
    }
    #endif /*LampPWM_CompareMode2SW */

#endif /* UseOneCompareMode */


#if (!LampPWM_UsingFixedFunction)


    /*******************************************************************************
    * Function Name: LampPWM_WriteCounter
    ********************************************************************************
    *
    * Summary:
    *  Writes a new counter value directly to the counter register. This will be
    *  implemented for that currently running period and only that period. This API
    *  is valid only for UDB implementation and not available for fixed function
    *  PWM implementation.
    *
    * Parameters:
    *  counter:  The period new period counter value.
    *
    * Return:
    *  None
    *
    * Side Effects:
    *  The PWM Period will be reloaded when a counter value will be a zero
    *
    *******************************************************************************/
    void LampPWM_WriteCounter(uint8 counter) \
                                       
    {
        CY_SET_REG8(LampPWM_COUNTER_LSB_PTR, counter);
    }


    /*******************************************************************************
    * Function Name: LampPWM_ReadCounter
    ********************************************************************************
    *
    * Summary:
    *  This function returns the current value of the counter.  It doesn't matter
    *  if the counter is enabled or running.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  The current value of the counter.
    *
    *******************************************************************************/
    uint8 LampPWM_ReadCounter(void) 
    {
        /* Force capture by reading Accumulator */
        /* Must first do a software capture to be able to read the counter */
        /* It is up to the user code to make sure there isn't already captured data in the FIFO */
          (void)CY_GET_REG8(LampPWM_COUNTERCAP_LSB_PTR_8BIT);

        /* Read the data from the FIFO */
        return (CY_GET_REG8(LampPWM_CAPTURE_LSB_PTR));
    }

    #if (LampPWM_UseStatus)


        /*******************************************************************************
        * Function Name: LampPWM_ClearFIFO
        ********************************************************************************
        *
        * Summary:
        *  This function clears all capture data from the capture FIFO
        *
        * Parameters:
        *  None
        *
        * Return:
        *  None
        *
        *******************************************************************************/
        void LampPWM_ClearFIFO(void) 
        {
            while(0u != (LampPWM_ReadStatusRegister() & LampPWM_STATUS_FIFONEMPTY))
            {
                (void)LampPWM_ReadCapture();
            }
        }

    #endif /* LampPWM_UseStatus */

#endif /* !LampPWM_UsingFixedFunction */


/*******************************************************************************
* Function Name: LampPWM_WritePeriod
********************************************************************************
*
* Summary:
*  This function is used to change the period of the counter.  The new period
*  will be loaded the next time terminal count is detected.
*
* Parameters:
*  period:  Period value. May be between 1 and (2^Resolution)-1.  A value of 0
*           will result in the counter remaining at zero.
*
* Return:
*  None
*
*******************************************************************************/
void LampPWM_WritePeriod(uint8 period) 
{
    #if(LampPWM_UsingFixedFunction)
        CY_SET_REG16(LampPWM_PERIOD_LSB_PTR, (uint16)period);
    #else
        CY_SET_REG8(LampPWM_PERIOD_LSB_PTR, period);
    #endif /* (LampPWM_UsingFixedFunction) */
}

#if (LampPWM_UseOneCompareMode)


    /*******************************************************************************
    * Function Name: LampPWM_WriteCompare
    ********************************************************************************
    *
    * Summary:
    *  This funtion is used to change the compare1 value when the PWM is in Dither
    *  mode. The compare output will reflect the new value on the next UDB clock.
    *  The compare output will be driven high when the present counter value is
    *  compared to the compare value based on the compare mode defined in
    *  Dither Mode.
    *
    * Parameters:
    *  compare:  New compare value.
    *
    * Return:
    *  None
    *
    * Side Effects:
    *  This function is only available if the PWM mode parameter is set to
    *  Dither Mode, Center Aligned Mode or One Output Mode
    *
    *******************************************************************************/
    void LampPWM_WriteCompare(uint8 compare) \
                                       
    {
        #if(LampPWM_UsingFixedFunction)
            CY_SET_REG16(LampPWM_COMPARE1_LSB_PTR, (uint16)compare);
        #else
            CY_SET_REG8(LampPWM_COMPARE1_LSB_PTR, compare);
        #endif /* (LampPWM_UsingFixedFunction) */

        #if (LampPWM_PWMMode == LampPWM__B_PWM__DITHER)
            #if(LampPWM_UsingFixedFunction)
                CY_SET_REG16(LampPWM_COMPARE2_LSB_PTR, (uint16)(compare + 1u));
            #else
                CY_SET_REG8(LampPWM_COMPARE2_LSB_PTR, (compare + 1u));
            #endif /* (LampPWM_UsingFixedFunction) */
        #endif /* (LampPWM_PWMMode == LampPWM__B_PWM__DITHER) */
    }


#else


    /*******************************************************************************
    * Function Name: LampPWM_WriteCompare1
    ********************************************************************************
    *
    * Summary:
    *  This funtion is used to change the compare1 value.  The compare output will
    *  reflect the new value on the next UDB clock.  The compare output will be
    *  driven high when the present counter value is less than or less than or
    *  equal to the compare register, depending on the mode.
    *
    * Parameters:
    *  compare:  New compare value.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void LampPWM_WriteCompare1(uint8 compare) \
                                        
    {
        #if(LampPWM_UsingFixedFunction)
            CY_SET_REG16(LampPWM_COMPARE1_LSB_PTR, (uint16)compare);
        #else
            CY_SET_REG8(LampPWM_COMPARE1_LSB_PTR, compare);
        #endif /* (LampPWM_UsingFixedFunction) */
    }


    /*******************************************************************************
    * Function Name: LampPWM_WriteCompare2
    ********************************************************************************
    *
    * Summary:
    *  This funtion is used to change the compare value, for compare1 output.
    *  The compare output will reflect the new value on the next UDB clock.
    *  The compare output will be driven high when the present counter value is
    *  less than or less than or equal to the compare register, depending on the
    *  mode.
    *
    * Parameters:
    *  compare:  New compare value.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void LampPWM_WriteCompare2(uint8 compare) \
                                        
    {
        #if(LampPWM_UsingFixedFunction)
            CY_SET_REG16(LampPWM_COMPARE2_LSB_PTR, compare);
        #else
            CY_SET_REG8(LampPWM_COMPARE2_LSB_PTR, compare);
        #endif /* (LampPWM_UsingFixedFunction) */
    }
#endif /* UseOneCompareMode */

#if (LampPWM_DeadBandUsed)


    /*******************************************************************************
    * Function Name: LampPWM_WriteDeadTime
    ********************************************************************************
    *
    * Summary:
    *  This function writes the dead-band counts to the corresponding register
    *
    * Parameters:
    *  deadtime:  Number of counts for dead time
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void LampPWM_WriteDeadTime(uint8 deadtime) 
    {
        /* If using the Dead Band 1-255 mode then just write the register */
        #if(!LampPWM_DeadBand2_4)
            CY_SET_REG8(LampPWM_DEADBAND_COUNT_PTR, deadtime);
        #else
            /* Otherwise the data has to be masked and offset */
            /* Clear existing data */
            LampPWM_DEADBAND_COUNT &= ((uint8)(~LampPWM_DEADBAND_COUNT_MASK));

            /* Set new dead time */
            #if(LampPWM_DEADBAND_COUNT_SHIFT)
                LampPWM_DEADBAND_COUNT |= ((uint8)((uint8)deadtime << LampPWM_DEADBAND_COUNT_SHIFT)) &
                                                    LampPWM_DEADBAND_COUNT_MASK;
            #else
                LampPWM_DEADBAND_COUNT |= deadtime & LampPWM_DEADBAND_COUNT_MASK;
            #endif /* (LampPWM_DEADBAND_COUNT_SHIFT) */

        #endif /* (!LampPWM_DeadBand2_4) */
    }


    /*******************************************************************************
    * Function Name: LampPWM_ReadDeadTime
    ********************************************************************************
    *
    * Summary:
    *  This function reads the dead-band counts from the corresponding register
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Dead Band Counts
    *
    *******************************************************************************/
    uint8 LampPWM_ReadDeadTime(void) 
    {
        /* If using the Dead Band 1-255 mode then just read the register */
        #if(!LampPWM_DeadBand2_4)
            return (CY_GET_REG8(LampPWM_DEADBAND_COUNT_PTR));
        #else

            /* Otherwise the data has to be masked and offset */
            #if(LampPWM_DEADBAND_COUNT_SHIFT)
                return ((uint8)(((uint8)(LampPWM_DEADBAND_COUNT & LampPWM_DEADBAND_COUNT_MASK)) >>
                                                                           LampPWM_DEADBAND_COUNT_SHIFT));
            #else
                return (LampPWM_DEADBAND_COUNT & LampPWM_DEADBAND_COUNT_MASK);
            #endif /* (LampPWM_DEADBAND_COUNT_SHIFT) */
        #endif /* (!LampPWM_DeadBand2_4) */
    }
#endif /* DeadBandUsed */

#if (LampPWM_UseStatus || LampPWM_UsingFixedFunction)


    /*******************************************************************************
    * Function Name: LampPWM_SetInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  This function configures the interrupts mask control of theinterrupt
    *  source status register.
    *
    * Parameters:
    *  uint8 interruptMode: Bit field containing the interrupt sources enabled
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void LampPWM_SetInterruptMode(uint8 interruptMode) 
    {
        CY_SET_REG8(LampPWM_STATUS_MASK_PTR, interruptMode);
    }


    /*******************************************************************************
    * Function Name: LampPWM_ReadStatusRegister
    ********************************************************************************
    *
    * Summary:
    *  This function returns the current state of the status register.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8 : Current status register value. The status register bits are:
    *  [7:6] : Unused(0)
    *  [5]   : Kill event output
    *  [4]   : FIFO not empty
    *  [3]   : FIFO full
    *  [2]   : Terminal count
    *  [1]   : Compare output 2
    *  [0]   : Compare output 1
    *
    *******************************************************************************/
    uint8 LampPWM_ReadStatusRegister(void) 
    {
        return (CY_GET_REG8(LampPWM_STATUS_PTR));
    }

#endif /* (LampPWM_UseStatus || LampPWM_UsingFixedFunction) */


#if (LampPWM_UseControl)


    /*******************************************************************************
    * Function Name: LampPWM_ReadControlRegister
    ********************************************************************************
    *
    * Summary:
    *  Returns the current state of the control register. This API is available
    *  only if the control register is not removed.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8 : Current control register value
    *
    *******************************************************************************/
    uint8 LampPWM_ReadControlRegister(void) 
    {
        uint8 result;

        result = CY_GET_REG8(LampPWM_CONTROL_PTR);
        return (result);
    }


    /*******************************************************************************
    * Function Name: LampPWM_WriteControlRegister
    ********************************************************************************
    *
    * Summary:
    *  Sets the bit field of the control register. This API is available only if
    *  the control register is not removed.
    *
    * Parameters:
    *  uint8 control: Control register bit field, The status register bits are:
    *  [7]   : PWM Enable
    *  [6]   : Reset
    *  [5:3] : Compare Mode2
    *  [2:0] : Compare Mode2
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void LampPWM_WriteControlRegister(uint8 control) 
    {
        CY_SET_REG8(LampPWM_CONTROL_PTR, control);
    }

#endif /* (LampPWM_UseControl) */


#if (!LampPWM_UsingFixedFunction)


    /*******************************************************************************
    * Function Name: LampPWM_ReadCapture
    ********************************************************************************
    *
    * Summary:
    *  Reads the capture value from the capture FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8/uint16: The current capture value
    *
    *******************************************************************************/
    uint8 LampPWM_ReadCapture(void) 
    {
        return (CY_GET_REG8(LampPWM_CAPTURE_LSB_PTR));
    }

#endif /* (!LampPWM_UsingFixedFunction) */


#if (LampPWM_UseOneCompareMode)


    /*******************************************************************************
    * Function Name: LampPWM_ReadCompare
    ********************************************************************************
    *
    * Summary:
    *  Reads the compare value for the compare output when the PWM Mode parameter is
    *  set to Dither mode, Center Aligned mode, or One Output mode.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8/uint16: Current compare value
    *
    *******************************************************************************/
    uint8 LampPWM_ReadCompare(void) 
    {
        #if(LampPWM_UsingFixedFunction)
            return ((uint8)CY_GET_REG16(LampPWM_COMPARE1_LSB_PTR));
        #else
            return (CY_GET_REG8(LampPWM_COMPARE1_LSB_PTR));
        #endif /* (LampPWM_UsingFixedFunction) */
    }

#else


    /*******************************************************************************
    * Function Name: LampPWM_ReadCompare1
    ********************************************************************************
    *
    * Summary:
    *  Reads the compare value for the compare1 output.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8/uint16: Current compare value.
    *
    *******************************************************************************/
    uint8 LampPWM_ReadCompare1(void) 
    {
        return (CY_GET_REG8(LampPWM_COMPARE1_LSB_PTR));
    }


    /*******************************************************************************
    * Function Name: LampPWM_ReadCompare2
    ********************************************************************************
    *
    * Summary:
    *  Reads the compare value for the compare2 output.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8/uint16: Current compare value.
    *
    *******************************************************************************/
    uint8 LampPWM_ReadCompare2(void) 
    {
        return (CY_GET_REG8(LampPWM_COMPARE2_LSB_PTR));
    }

#endif /* (LampPWM_UseOneCompareMode) */


/*******************************************************************************
* Function Name: LampPWM_ReadPeriod
********************************************************************************
*
* Summary:
*  Reads the period value used by the PWM hardware.
*
* Parameters:
*  None
*
* Return:
*  uint8/16: Period value
*
*******************************************************************************/
uint8 LampPWM_ReadPeriod(void) 
{
    #if(LampPWM_UsingFixedFunction)
        return ((uint8)CY_GET_REG16(LampPWM_PERIOD_LSB_PTR));
    #else
        return (CY_GET_REG8(LampPWM_PERIOD_LSB_PTR));
    #endif /* (LampPWM_UsingFixedFunction) */
}

#if ( LampPWM_KillModeMinTime)


    /*******************************************************************************
    * Function Name: LampPWM_WriteKillTime
    ********************************************************************************
    *
    * Summary:
    *  Writes the kill time value used by the hardware when the Kill Mode
    *  is set to Minimum Time.
    *
    * Parameters:
    *  uint8: Minimum Time kill counts
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void LampPWM_WriteKillTime(uint8 killtime) 
    {
        CY_SET_REG8(LampPWM_KILLMODEMINTIME_PTR, killtime);
    }


    /*******************************************************************************
    * Function Name: LampPWM_ReadKillTime
    ********************************************************************************
    *
    * Summary:
    *  Reads the kill time value used by the hardware when the Kill Mode is set
    *  to Minimum Time.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8: The current Minimum Time kill counts
    *
    *******************************************************************************/
    uint8 LampPWM_ReadKillTime(void) 
    {
        return (CY_GET_REG8(LampPWM_KILLMODEMINTIME_PTR));
    }

#endif /* ( LampPWM_KillModeMinTime) */

/* [] END OF FILE */
