/*******************************************************************************
* File Name: LampPWM_PM.c
* Version 3.30
*
* Description:
*  This file provides the power management source code to API for the
*  PWM.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "LampPWM.h"

static LampPWM_backupStruct LampPWM_backup;


/*******************************************************************************
* Function Name: LampPWM_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  LampPWM_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void LampPWM_SaveConfig(void) 
{

    #if(!LampPWM_UsingFixedFunction)
        #if(!LampPWM_PWMModeIsCenterAligned)
            LampPWM_backup.PWMPeriod = LampPWM_ReadPeriod();
        #endif /* (!LampPWM_PWMModeIsCenterAligned) */
        LampPWM_backup.PWMUdb = LampPWM_ReadCounter();
        #if (LampPWM_UseStatus)
            LampPWM_backup.InterruptMaskValue = LampPWM_STATUS_MASK;
        #endif /* (LampPWM_UseStatus) */

        #if(LampPWM_DeadBandMode == LampPWM__B_PWM__DBM_256_CLOCKS || \
            LampPWM_DeadBandMode == LampPWM__B_PWM__DBM_2_4_CLOCKS)
            LampPWM_backup.PWMdeadBandValue = LampPWM_ReadDeadTime();
        #endif /*  deadband count is either 2-4 clocks or 256 clocks */

        #if(LampPWM_KillModeMinTime)
             LampPWM_backup.PWMKillCounterPeriod = LampPWM_ReadKillTime();
        #endif /* (LampPWM_KillModeMinTime) */

        #if(LampPWM_UseControl)
            LampPWM_backup.PWMControlRegister = LampPWM_ReadControlRegister();
        #endif /* (LampPWM_UseControl) */
    #endif  /* (!LampPWM_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: LampPWM_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  LampPWM_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void LampPWM_RestoreConfig(void) 
{
        #if(!LampPWM_UsingFixedFunction)
            #if(!LampPWM_PWMModeIsCenterAligned)
                LampPWM_WritePeriod(LampPWM_backup.PWMPeriod);
            #endif /* (!LampPWM_PWMModeIsCenterAligned) */

            LampPWM_WriteCounter(LampPWM_backup.PWMUdb);

            #if (LampPWM_UseStatus)
                LampPWM_STATUS_MASK = LampPWM_backup.InterruptMaskValue;
            #endif /* (LampPWM_UseStatus) */

            #if(LampPWM_DeadBandMode == LampPWM__B_PWM__DBM_256_CLOCKS || \
                LampPWM_DeadBandMode == LampPWM__B_PWM__DBM_2_4_CLOCKS)
                LampPWM_WriteDeadTime(LampPWM_backup.PWMdeadBandValue);
            #endif /* deadband count is either 2-4 clocks or 256 clocks */

            #if(LampPWM_KillModeMinTime)
                LampPWM_WriteKillTime(LampPWM_backup.PWMKillCounterPeriod);
            #endif /* (LampPWM_KillModeMinTime) */

            #if(LampPWM_UseControl)
                LampPWM_WriteControlRegister(LampPWM_backup.PWMControlRegister);
            #endif /* (LampPWM_UseControl) */
        #endif  /* (!LampPWM_UsingFixedFunction) */
    }


/*******************************************************************************
* Function Name: LampPWM_Sleep
********************************************************************************
*
* Summary:
*  Disables block's operation and saves the user configuration. Should be called
*  just prior to entering sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  LampPWM_backup.PWMEnableState:  Is modified depending on the enable
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void LampPWM_Sleep(void) 
{
    #if(LampPWM_UseControl)
        if(LampPWM_CTRL_ENABLE == (LampPWM_CONTROL & LampPWM_CTRL_ENABLE))
        {
            /*Component is enabled */
            LampPWM_backup.PWMEnableState = 1u;
        }
        else
        {
            /* Component is disabled */
            LampPWM_backup.PWMEnableState = 0u;
        }
    #endif /* (LampPWM_UseControl) */

    /* Stop component */
    LampPWM_Stop();

    /* Save registers configuration */
    LampPWM_SaveConfig();
}


/*******************************************************************************
* Function Name: LampPWM_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration. Should be called just after
*  awaking from sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  LampPWM_backup.pwmEnable:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void LampPWM_Wakeup(void) 
{
     /* Restore registers values */
    LampPWM_RestoreConfig();

    if(LampPWM_backup.PWMEnableState != 0u)
    {
        /* Enable component's operation */
        LampPWM_Enable();
    } /* Do nothing if component's block was disabled before */

}


/* [] END OF FILE */
