/*******************************************************************************
* File Name: PinF.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "PinF.h"

static PinF_BACKUP_STRUCT  PinF_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: PinF_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet PinF_SUT.c usage_PinF_Sleep_Wakeup
*******************************************************************************/
void PinF_Sleep(void)
{
    #if defined(PinF__PC)
        PinF_backup.pcState = PinF_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            PinF_backup.usbState = PinF_CR1_REG;
            PinF_USB_POWER_REG |= PinF_USBIO_ENTER_SLEEP;
            PinF_CR1_REG &= PinF_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(PinF__SIO)
        PinF_backup.sioState = PinF_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        PinF_SIO_REG &= (uint32)(~PinF_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: PinF_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to PinF_Sleep() for an example usage.
*******************************************************************************/
void PinF_Wakeup(void)
{
    #if defined(PinF__PC)
        PinF_PC = PinF_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            PinF_USB_POWER_REG &= PinF_USBIO_EXIT_SLEEP_PH1;
            PinF_CR1_REG = PinF_backup.usbState;
            PinF_USB_POWER_REG &= PinF_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(PinF__SIO)
        PinF_SIO_REG = PinF_backup.sioState;
    #endif
}


/* [] END OF FILE */
