/*******************************************************************************
* File Name: PinPWM.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "PinPWM.h"

static PinPWM_BACKUP_STRUCT  PinPWM_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: PinPWM_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet PinPWM_SUT.c usage_PinPWM_Sleep_Wakeup
*******************************************************************************/
void PinPWM_Sleep(void)
{
    #if defined(PinPWM__PC)
        PinPWM_backup.pcState = PinPWM_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            PinPWM_backup.usbState = PinPWM_CR1_REG;
            PinPWM_USB_POWER_REG |= PinPWM_USBIO_ENTER_SLEEP;
            PinPWM_CR1_REG &= PinPWM_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(PinPWM__SIO)
        PinPWM_backup.sioState = PinPWM_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        PinPWM_SIO_REG &= (uint32)(~PinPWM_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: PinPWM_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to PinPWM_Sleep() for an example usage.
*******************************************************************************/
void PinPWM_Wakeup(void)
{
    #if defined(PinPWM__PC)
        PinPWM_PC = PinPWM_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            PinPWM_USB_POWER_REG &= PinPWM_USBIO_EXIT_SLEEP_PH1;
            PinPWM_CR1_REG = PinPWM_backup.usbState;
            PinPWM_USB_POWER_REG &= PinPWM_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(PinPWM__SIO)
        PinPWM_SIO_REG = PinPWM_backup.sioState;
    #endif
}


/* [] END OF FILE */
