/*******************************************************************************
* File Name: pwmGenerator_PM.c
* Version 3.30
*
* Description:
*  This file provides the power management source code to API for the
*  PWM.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "pwmGenerator.h"

static pwmGenerator_backupStruct pwmGenerator_backup;


/*******************************************************************************
* Function Name: pwmGenerator_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  pwmGenerator_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void pwmGenerator_SaveConfig(void) 
{

    #if(!pwmGenerator_UsingFixedFunction)
        #if(!pwmGenerator_PWMModeIsCenterAligned)
            pwmGenerator_backup.PWMPeriod = pwmGenerator_ReadPeriod();
        #endif /* (!pwmGenerator_PWMModeIsCenterAligned) */
        pwmGenerator_backup.PWMUdb = pwmGenerator_ReadCounter();
        #if (pwmGenerator_UseStatus)
            pwmGenerator_backup.InterruptMaskValue = pwmGenerator_STATUS_MASK;
        #endif /* (pwmGenerator_UseStatus) */

        #if(pwmGenerator_DeadBandMode == pwmGenerator__B_PWM__DBM_256_CLOCKS || \
            pwmGenerator_DeadBandMode == pwmGenerator__B_PWM__DBM_2_4_CLOCKS)
            pwmGenerator_backup.PWMdeadBandValue = pwmGenerator_ReadDeadTime();
        #endif /*  deadband count is either 2-4 clocks or 256 clocks */

        #if(pwmGenerator_KillModeMinTime)
             pwmGenerator_backup.PWMKillCounterPeriod = pwmGenerator_ReadKillTime();
        #endif /* (pwmGenerator_KillModeMinTime) */

        #if(pwmGenerator_UseControl)
            pwmGenerator_backup.PWMControlRegister = pwmGenerator_ReadControlRegister();
        #endif /* (pwmGenerator_UseControl) */
    #endif  /* (!pwmGenerator_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: pwmGenerator_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  pwmGenerator_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void pwmGenerator_RestoreConfig(void) 
{
        #if(!pwmGenerator_UsingFixedFunction)
            #if(!pwmGenerator_PWMModeIsCenterAligned)
                pwmGenerator_WritePeriod(pwmGenerator_backup.PWMPeriod);
            #endif /* (!pwmGenerator_PWMModeIsCenterAligned) */

            pwmGenerator_WriteCounter(pwmGenerator_backup.PWMUdb);

            #if (pwmGenerator_UseStatus)
                pwmGenerator_STATUS_MASK = pwmGenerator_backup.InterruptMaskValue;
            #endif /* (pwmGenerator_UseStatus) */

            #if(pwmGenerator_DeadBandMode == pwmGenerator__B_PWM__DBM_256_CLOCKS || \
                pwmGenerator_DeadBandMode == pwmGenerator__B_PWM__DBM_2_4_CLOCKS)
                pwmGenerator_WriteDeadTime(pwmGenerator_backup.PWMdeadBandValue);
            #endif /* deadband count is either 2-4 clocks or 256 clocks */

            #if(pwmGenerator_KillModeMinTime)
                pwmGenerator_WriteKillTime(pwmGenerator_backup.PWMKillCounterPeriod);
            #endif /* (pwmGenerator_KillModeMinTime) */

            #if(pwmGenerator_UseControl)
                pwmGenerator_WriteControlRegister(pwmGenerator_backup.PWMControlRegister);
            #endif /* (pwmGenerator_UseControl) */
        #endif  /* (!pwmGenerator_UsingFixedFunction) */
    }


/*******************************************************************************
* Function Name: pwmGenerator_Sleep
********************************************************************************
*
* Summary:
*  Disables block's operation and saves the user configuration. Should be called
*  just prior to entering sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  pwmGenerator_backup.PWMEnableState:  Is modified depending on the enable
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void pwmGenerator_Sleep(void) 
{
    #if(pwmGenerator_UseControl)
        if(pwmGenerator_CTRL_ENABLE == (pwmGenerator_CONTROL & pwmGenerator_CTRL_ENABLE))
        {
            /*Component is enabled */
            pwmGenerator_backup.PWMEnableState = 1u;
        }
        else
        {
            /* Component is disabled */
            pwmGenerator_backup.PWMEnableState = 0u;
        }
    #endif /* (pwmGenerator_UseControl) */

    /* Stop component */
    pwmGenerator_Stop();

    /* Save registers configuration */
    pwmGenerator_SaveConfig();
}


/*******************************************************************************
* Function Name: pwmGenerator_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration. Should be called just after
*  awaking from sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  pwmGenerator_backup.pwmEnable:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void pwmGenerator_Wakeup(void) 
{
     /* Restore registers values */
    pwmGenerator_RestoreConfig();

    if(pwmGenerator_backup.PWMEnableState != 0u)
    {
        /* Enable component's operation */
        pwmGenerator_Enable();
    } /* Do nothing if component's block was disabled before */

}


/* [] END OF FILE */
