/*******************************************************************************
* File Name: pwmGenerator.c
* Version 3.30
*
* Description:
*  The PWM User Module consist of an 8 or 16-bit counter with two 8 or 16-bit
*  comparitors. Each instance of this user module is capable of generating
*  two PWM outputs with the same period. The pulse width is selectable between
*  1 and 255/65535. The period is selectable between 2 and 255/65536 clocks.
*  The compare value output may be configured to be active when the present
*  counter is less than or less than/equal to the compare value.
*  A terminal count output is also provided. It generates a pulse one clock
*  width wide when the counter is equal to zero.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "pwmGenerator.h"

/* Error message for removed <resource> through optimization */
#ifdef pwmGenerator_PWMUDB_genblk1_ctrlreg__REMOVED
    #error PWM_v3_30 detected with a constant 0 for the enable or \
         constant 1 for reset. This will prevent the component from operating.
#endif /* pwmGenerator_PWMUDB_genblk1_ctrlreg__REMOVED */

uint8 pwmGenerator_initVar = 0u;


/*******************************************************************************
* Function Name: pwmGenerator_Start
********************************************************************************
*
* Summary:
*  The start function initializes the pwm with the default values, the
*  enables the counter to begin counting.  It does not enable interrupts,
*  the EnableInt command should be called if interrupt generation is required.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  pwmGenerator_initVar: Is modified when this function is called for the
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void pwmGenerator_Start(void) 
{
    /* If not Initialized then initialize all required hardware and software */
    if(pwmGenerator_initVar == 0u)
    {
        pwmGenerator_Init();
        pwmGenerator_initVar = 1u;
    }
    pwmGenerator_Enable();

}


/*******************************************************************************
* Function Name: pwmGenerator_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the
*  customizer of the component placed onto schematic. Usually called in
*  pwmGenerator_Start().
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void pwmGenerator_Init(void) 
{
    #if (pwmGenerator_UsingFixedFunction || pwmGenerator_UseControl)
        uint8 ctrl;
    #endif /* (pwmGenerator_UsingFixedFunction || pwmGenerator_UseControl) */

    #if(!pwmGenerator_UsingFixedFunction)
        #if(pwmGenerator_UseStatus)
            /* Interrupt State Backup for Critical Region*/
            uint8 pwmGenerator_interruptState;
        #endif /* (pwmGenerator_UseStatus) */
    #endif /* (!pwmGenerator_UsingFixedFunction) */

    #if (pwmGenerator_UsingFixedFunction)
        /* You are allowed to write the compare value (FF only) */
        pwmGenerator_CONTROL |= pwmGenerator_CFG0_MODE;
        #if (pwmGenerator_DeadBand2_4)
            pwmGenerator_CONTROL |= pwmGenerator_CFG0_DB;
        #endif /* (pwmGenerator_DeadBand2_4) */

        ctrl = pwmGenerator_CONTROL3 & ((uint8 )(~pwmGenerator_CTRL_CMPMODE1_MASK));
        pwmGenerator_CONTROL3 = ctrl | pwmGenerator_DEFAULT_COMPARE1_MODE;

         /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
        pwmGenerator_RT1 &= ((uint8)(~pwmGenerator_RT1_MASK));
        pwmGenerator_RT1 |= pwmGenerator_SYNC;

        /*Enable DSI Sync all all inputs of the PWM*/
        pwmGenerator_RT1 &= ((uint8)(~pwmGenerator_SYNCDSI_MASK));
        pwmGenerator_RT1 |= pwmGenerator_SYNCDSI_EN;

    #elif (pwmGenerator_UseControl)
        /* Set the default compare mode defined in the parameter */
        ctrl = pwmGenerator_CONTROL & ((uint8)(~pwmGenerator_CTRL_CMPMODE2_MASK)) &
                ((uint8)(~pwmGenerator_CTRL_CMPMODE1_MASK));
        pwmGenerator_CONTROL = ctrl | pwmGenerator_DEFAULT_COMPARE2_MODE |
                                   pwmGenerator_DEFAULT_COMPARE1_MODE;
    #endif /* (pwmGenerator_UsingFixedFunction) */

    #if (!pwmGenerator_UsingFixedFunction)
        #if (pwmGenerator_Resolution == 8)
            /* Set FIFO 0 to 1 byte register for period*/
            pwmGenerator_AUX_CONTROLDP0 |= (pwmGenerator_AUX_CTRL_FIFO0_CLR);
        #else /* (pwmGenerator_Resolution == 16)*/
            /* Set FIFO 0 to 1 byte register for period */
            pwmGenerator_AUX_CONTROLDP0 |= (pwmGenerator_AUX_CTRL_FIFO0_CLR);
            pwmGenerator_AUX_CONTROLDP1 |= (pwmGenerator_AUX_CTRL_FIFO0_CLR);
        #endif /* (pwmGenerator_Resolution == 8) */

        pwmGenerator_WriteCounter(pwmGenerator_INIT_PERIOD_VALUE);
    #endif /* (!pwmGenerator_UsingFixedFunction) */

    pwmGenerator_WritePeriod(pwmGenerator_INIT_PERIOD_VALUE);

        #if (pwmGenerator_UseOneCompareMode)
            pwmGenerator_WriteCompare(pwmGenerator_INIT_COMPARE_VALUE1);
        #else
            pwmGenerator_WriteCompare1(pwmGenerator_INIT_COMPARE_VALUE1);
            pwmGenerator_WriteCompare2(pwmGenerator_INIT_COMPARE_VALUE2);
        #endif /* (pwmGenerator_UseOneCompareMode) */

        #if (pwmGenerator_KillModeMinTime)
            pwmGenerator_WriteKillTime(pwmGenerator_MinimumKillTime);
        #endif /* (pwmGenerator_KillModeMinTime) */

        #if (pwmGenerator_DeadBandUsed)
            pwmGenerator_WriteDeadTime(pwmGenerator_INIT_DEAD_TIME);
        #endif /* (pwmGenerator_DeadBandUsed) */

    #if (pwmGenerator_UseStatus || pwmGenerator_UsingFixedFunction)
        pwmGenerator_SetInterruptMode(pwmGenerator_INIT_INTERRUPTS_MODE);
    #endif /* (pwmGenerator_UseStatus || pwmGenerator_UsingFixedFunction) */

    #if (pwmGenerator_UsingFixedFunction)
        /* Globally Enable the Fixed Function Block chosen */
        pwmGenerator_GLOBAL_ENABLE |= pwmGenerator_BLOCK_EN_MASK;
        /* Set the Interrupt source to come from the status register */
        pwmGenerator_CONTROL2 |= pwmGenerator_CTRL2_IRQ_SEL;
    #else
        #if(pwmGenerator_UseStatus)

            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            pwmGenerator_interruptState = CyEnterCriticalSection();
            /* Use the interrupt output of the status register for IRQ output */
            pwmGenerator_STATUS_AUX_CTRL |= pwmGenerator_STATUS_ACTL_INT_EN_MASK;

             /* Exit Critical Region*/
            CyExitCriticalSection(pwmGenerator_interruptState);

            /* Clear the FIFO to enable the pwmGenerator_STATUS_FIFOFULL
                   bit to be set on FIFO full. */
            pwmGenerator_ClearFIFO();
        #endif /* (pwmGenerator_UseStatus) */
    #endif /* (pwmGenerator_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: pwmGenerator_Enable
********************************************************************************
*
* Summary:
*  Enables the PWM block operation
*
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  This works only if software enable mode is chosen
*
*******************************************************************************/
void pwmGenerator_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (pwmGenerator_UsingFixedFunction)
        pwmGenerator_GLOBAL_ENABLE |= pwmGenerator_BLOCK_EN_MASK;
        pwmGenerator_GLOBAL_STBY_ENABLE |= pwmGenerator_BLOCK_STBY_EN_MASK;
    #endif /* (pwmGenerator_UsingFixedFunction) */

    /* Enable the PWM from the control register  */
    #if (pwmGenerator_UseControl || pwmGenerator_UsingFixedFunction)
        pwmGenerator_CONTROL |= pwmGenerator_CTRL_ENABLE;
    #endif /* (pwmGenerator_UseControl || pwmGenerator_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: pwmGenerator_Stop
********************************************************************************
*
* Summary:
*  The stop function halts the PWM, but does not change any modes or disable
*  interrupts.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  If the Enable mode is set to Hardware only then this function
*  has no effect on the operation of the PWM
*
*******************************************************************************/
void pwmGenerator_Stop(void) 
{
    #if (pwmGenerator_UseControl || pwmGenerator_UsingFixedFunction)
        pwmGenerator_CONTROL &= ((uint8)(~pwmGenerator_CTRL_ENABLE));
    #endif /* (pwmGenerator_UseControl || pwmGenerator_UsingFixedFunction) */

    /* Globally disable the Fixed Function Block chosen */
    #if (pwmGenerator_UsingFixedFunction)
        pwmGenerator_GLOBAL_ENABLE &= ((uint8)(~pwmGenerator_BLOCK_EN_MASK));
        pwmGenerator_GLOBAL_STBY_ENABLE &= ((uint8)(~pwmGenerator_BLOCK_STBY_EN_MASK));
    #endif /* (pwmGenerator_UsingFixedFunction) */
}

#if (pwmGenerator_UseOneCompareMode)
    #if (pwmGenerator_CompareMode1SW)


        /*******************************************************************************
        * Function Name: pwmGenerator_SetCompareMode
        ********************************************************************************
        *
        * Summary:
        *  This function writes the Compare Mode for the pwm output when in Dither mode,
        *  Center Align Mode or One Output Mode.
        *
        * Parameters:
        *  comparemode:  The new compare mode for the PWM output. Use the compare types
        *                defined in the H file as input arguments.
        *
        * Return:
        *  None
        *
        *******************************************************************************/
        void pwmGenerator_SetCompareMode(uint8 comparemode) 
        {
            #if(pwmGenerator_UsingFixedFunction)

                #if(0 != pwmGenerator_CTRL_CMPMODE1_SHIFT)
                    uint8 comparemodemasked = ((uint8)((uint8)comparemode << pwmGenerator_CTRL_CMPMODE1_SHIFT));
                #else
                    uint8 comparemodemasked = comparemode;
                #endif /* (0 != pwmGenerator_CTRL_CMPMODE1_SHIFT) */

                pwmGenerator_CONTROL3 &= ((uint8)(~pwmGenerator_CTRL_CMPMODE1_MASK)); /*Clear Existing Data */
                pwmGenerator_CONTROL3 |= comparemodemasked;

            #elif (pwmGenerator_UseControl)

                #if(0 != pwmGenerator_CTRL_CMPMODE1_SHIFT)
                    uint8 comparemode1masked = ((uint8)((uint8)comparemode << pwmGenerator_CTRL_CMPMODE1_SHIFT)) &
                                                pwmGenerator_CTRL_CMPMODE1_MASK;
                #else
                    uint8 comparemode1masked = comparemode & pwmGenerator_CTRL_CMPMODE1_MASK;
                #endif /* (0 != pwmGenerator_CTRL_CMPMODE1_SHIFT) */

                #if(0 != pwmGenerator_CTRL_CMPMODE2_SHIFT)
                    uint8 comparemode2masked = ((uint8)((uint8)comparemode << pwmGenerator_CTRL_CMPMODE2_SHIFT)) &
                                               pwmGenerator_CTRL_CMPMODE2_MASK;
                #else
                    uint8 comparemode2masked = comparemode & pwmGenerator_CTRL_CMPMODE2_MASK;
                #endif /* (0 != pwmGenerator_CTRL_CMPMODE2_SHIFT) */

                /*Clear existing mode */
                pwmGenerator_CONTROL &= ((uint8)(~(pwmGenerator_CTRL_CMPMODE1_MASK |
                                            pwmGenerator_CTRL_CMPMODE2_MASK)));
                pwmGenerator_CONTROL |= (comparemode1masked | comparemode2masked);

            #else
                uint8 temp = comparemode;
            #endif /* (pwmGenerator_UsingFixedFunction) */
        }
    #endif /* pwmGenerator_CompareMode1SW */

#else /* UseOneCompareMode */

    #if (pwmGenerator_CompareMode1SW)


        /*******************************************************************************
        * Function Name: pwmGenerator_SetCompareMode1
        ********************************************************************************
        *
        * Summary:
        *  This function writes the Compare Mode for the pwm or pwm1 output
        *
        * Parameters:
        *  comparemode:  The new compare mode for the PWM output. Use the compare types
        *                defined in the H file as input arguments.
        *
        * Return:
        *  None
        *
        *******************************************************************************/
        void pwmGenerator_SetCompareMode1(uint8 comparemode) 
        {
            #if(0 != pwmGenerator_CTRL_CMPMODE1_SHIFT)
                uint8 comparemodemasked = ((uint8)((uint8)comparemode << pwmGenerator_CTRL_CMPMODE1_SHIFT)) &
                                           pwmGenerator_CTRL_CMPMODE1_MASK;
            #else
                uint8 comparemodemasked = comparemode & pwmGenerator_CTRL_CMPMODE1_MASK;
            #endif /* (0 != pwmGenerator_CTRL_CMPMODE1_SHIFT) */

            #if (pwmGenerator_UseControl)
                pwmGenerator_CONTROL &= ((uint8)(~pwmGenerator_CTRL_CMPMODE1_MASK)); /*Clear existing mode */
                pwmGenerator_CONTROL |= comparemodemasked;
            #endif /* (pwmGenerator_UseControl) */
        }
    #endif /* pwmGenerator_CompareMode1SW */

#if (pwmGenerator_CompareMode2SW)


    /*******************************************************************************
    * Function Name: pwmGenerator_SetCompareMode2
    ********************************************************************************
    *
    * Summary:
    *  This function writes the Compare Mode for the pwm or pwm2 output
    *
    * Parameters:
    *  comparemode:  The new compare mode for the PWM output. Use the compare types
    *                defined in the H file as input arguments.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void pwmGenerator_SetCompareMode2(uint8 comparemode) 
    {

        #if(0 != pwmGenerator_CTRL_CMPMODE2_SHIFT)
            uint8 comparemodemasked = ((uint8)((uint8)comparemode << pwmGenerator_CTRL_CMPMODE2_SHIFT)) &
                                                 pwmGenerator_CTRL_CMPMODE2_MASK;
        #else
            uint8 comparemodemasked = comparemode & pwmGenerator_CTRL_CMPMODE2_MASK;
        #endif /* (0 != pwmGenerator_CTRL_CMPMODE2_SHIFT) */

        #if (pwmGenerator_UseControl)
            pwmGenerator_CONTROL &= ((uint8)(~pwmGenerator_CTRL_CMPMODE2_MASK)); /*Clear existing mode */
            pwmGenerator_CONTROL |= comparemodemasked;
        #endif /* (pwmGenerator_UseControl) */
    }
    #endif /*pwmGenerator_CompareMode2SW */

#endif /* UseOneCompareMode */


#if (!pwmGenerator_UsingFixedFunction)


    /*******************************************************************************
    * Function Name: pwmGenerator_WriteCounter
    ********************************************************************************
    *
    * Summary:
    *  Writes a new counter value directly to the counter register. This will be
    *  implemented for that currently running period and only that period. This API
    *  is valid only for UDB implementation and not available for fixed function
    *  PWM implementation.
    *
    * Parameters:
    *  counter:  The period new period counter value.
    *
    * Return:
    *  None
    *
    * Side Effects:
    *  The PWM Period will be reloaded when a counter value will be a zero
    *
    *******************************************************************************/
    void pwmGenerator_WriteCounter(uint8 counter) \
                                       
    {
        CY_SET_REG8(pwmGenerator_COUNTER_LSB_PTR, counter);
    }


    /*******************************************************************************
    * Function Name: pwmGenerator_ReadCounter
    ********************************************************************************
    *
    * Summary:
    *  This function returns the current value of the counter.  It doesn't matter
    *  if the counter is enabled or running.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  The current value of the counter.
    *
    *******************************************************************************/
    uint8 pwmGenerator_ReadCounter(void) 
    {
        /* Force capture by reading Accumulator */
        /* Must first do a software capture to be able to read the counter */
        /* It is up to the user code to make sure there isn't already captured data in the FIFO */
          (void)CY_GET_REG8(pwmGenerator_COUNTERCAP_LSB_PTR_8BIT);

        /* Read the data from the FIFO */
        return (CY_GET_REG8(pwmGenerator_CAPTURE_LSB_PTR));
    }

    #if (pwmGenerator_UseStatus)


        /*******************************************************************************
        * Function Name: pwmGenerator_ClearFIFO
        ********************************************************************************
        *
        * Summary:
        *  This function clears all capture data from the capture FIFO
        *
        * Parameters:
        *  None
        *
        * Return:
        *  None
        *
        *******************************************************************************/
        void pwmGenerator_ClearFIFO(void) 
        {
            while(0u != (pwmGenerator_ReadStatusRegister() & pwmGenerator_STATUS_FIFONEMPTY))
            {
                (void)pwmGenerator_ReadCapture();
            }
        }

    #endif /* pwmGenerator_UseStatus */

#endif /* !pwmGenerator_UsingFixedFunction */


/*******************************************************************************
* Function Name: pwmGenerator_WritePeriod
********************************************************************************
*
* Summary:
*  This function is used to change the period of the counter.  The new period
*  will be loaded the next time terminal count is detected.
*
* Parameters:
*  period:  Period value. May be between 1 and (2^Resolution)-1.  A value of 0
*           will result in the counter remaining at zero.
*
* Return:
*  None
*
*******************************************************************************/
void pwmGenerator_WritePeriod(uint8 period) 
{
    #if(pwmGenerator_UsingFixedFunction)
        CY_SET_REG16(pwmGenerator_PERIOD_LSB_PTR, (uint16)period);
    #else
        CY_SET_REG8(pwmGenerator_PERIOD_LSB_PTR, period);
    #endif /* (pwmGenerator_UsingFixedFunction) */
}

#if (pwmGenerator_UseOneCompareMode)


    /*******************************************************************************
    * Function Name: pwmGenerator_WriteCompare
    ********************************************************************************
    *
    * Summary:
    *  This funtion is used to change the compare1 value when the PWM is in Dither
    *  mode. The compare output will reflect the new value on the next UDB clock.
    *  The compare output will be driven high when the present counter value is
    *  compared to the compare value based on the compare mode defined in
    *  Dither Mode.
    *
    * Parameters:
    *  compare:  New compare value.
    *
    * Return:
    *  None
    *
    * Side Effects:
    *  This function is only available if the PWM mode parameter is set to
    *  Dither Mode, Center Aligned Mode or One Output Mode
    *
    *******************************************************************************/
    void pwmGenerator_WriteCompare(uint8 compare) \
                                       
    {
        #if(pwmGenerator_UsingFixedFunction)
            CY_SET_REG16(pwmGenerator_COMPARE1_LSB_PTR, (uint16)compare);
        #else
            CY_SET_REG8(pwmGenerator_COMPARE1_LSB_PTR, compare);
        #endif /* (pwmGenerator_UsingFixedFunction) */

        #if (pwmGenerator_PWMMode == pwmGenerator__B_PWM__DITHER)
            #if(pwmGenerator_UsingFixedFunction)
                CY_SET_REG16(pwmGenerator_COMPARE2_LSB_PTR, (uint16)(compare + 1u));
            #else
                CY_SET_REG8(pwmGenerator_COMPARE2_LSB_PTR, (compare + 1u));
            #endif /* (pwmGenerator_UsingFixedFunction) */
        #endif /* (pwmGenerator_PWMMode == pwmGenerator__B_PWM__DITHER) */
    }


#else


    /*******************************************************************************
    * Function Name: pwmGenerator_WriteCompare1
    ********************************************************************************
    *
    * Summary:
    *  This funtion is used to change the compare1 value.  The compare output will
    *  reflect the new value on the next UDB clock.  The compare output will be
    *  driven high when the present counter value is less than or less than or
    *  equal to the compare register, depending on the mode.
    *
    * Parameters:
    *  compare:  New compare value.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void pwmGenerator_WriteCompare1(uint8 compare) \
                                        
    {
        #if(pwmGenerator_UsingFixedFunction)
            CY_SET_REG16(pwmGenerator_COMPARE1_LSB_PTR, (uint16)compare);
        #else
            CY_SET_REG8(pwmGenerator_COMPARE1_LSB_PTR, compare);
        #endif /* (pwmGenerator_UsingFixedFunction) */
    }


    /*******************************************************************************
    * Function Name: pwmGenerator_WriteCompare2
    ********************************************************************************
    *
    * Summary:
    *  This funtion is used to change the compare value, for compare1 output.
    *  The compare output will reflect the new value on the next UDB clock.
    *  The compare output will be driven high when the present counter value is
    *  less than or less than or equal to the compare register, depending on the
    *  mode.
    *
    * Parameters:
    *  compare:  New compare value.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void pwmGenerator_WriteCompare2(uint8 compare) \
                                        
    {
        #if(pwmGenerator_UsingFixedFunction)
            CY_SET_REG16(pwmGenerator_COMPARE2_LSB_PTR, compare);
        #else
            CY_SET_REG8(pwmGenerator_COMPARE2_LSB_PTR, compare);
        #endif /* (pwmGenerator_UsingFixedFunction) */
    }
#endif /* UseOneCompareMode */

#if (pwmGenerator_DeadBandUsed)


    /*******************************************************************************
    * Function Name: pwmGenerator_WriteDeadTime
    ********************************************************************************
    *
    * Summary:
    *  This function writes the dead-band counts to the corresponding register
    *
    * Parameters:
    *  deadtime:  Number of counts for dead time
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void pwmGenerator_WriteDeadTime(uint8 deadtime) 
    {
        /* If using the Dead Band 1-255 mode then just write the register */
        #if(!pwmGenerator_DeadBand2_4)
            CY_SET_REG8(pwmGenerator_DEADBAND_COUNT_PTR, deadtime);
        #else
            /* Otherwise the data has to be masked and offset */
            /* Clear existing data */
            pwmGenerator_DEADBAND_COUNT &= ((uint8)(~pwmGenerator_DEADBAND_COUNT_MASK));

            /* Set new dead time */
            #if(pwmGenerator_DEADBAND_COUNT_SHIFT)
                pwmGenerator_DEADBAND_COUNT |= ((uint8)((uint8)deadtime << pwmGenerator_DEADBAND_COUNT_SHIFT)) &
                                                    pwmGenerator_DEADBAND_COUNT_MASK;
            #else
                pwmGenerator_DEADBAND_COUNT |= deadtime & pwmGenerator_DEADBAND_COUNT_MASK;
            #endif /* (pwmGenerator_DEADBAND_COUNT_SHIFT) */

        #endif /* (!pwmGenerator_DeadBand2_4) */
    }


    /*******************************************************************************
    * Function Name: pwmGenerator_ReadDeadTime
    ********************************************************************************
    *
    * Summary:
    *  This function reads the dead-band counts from the corresponding register
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Dead Band Counts
    *
    *******************************************************************************/
    uint8 pwmGenerator_ReadDeadTime(void) 
    {
        /* If using the Dead Band 1-255 mode then just read the register */
        #if(!pwmGenerator_DeadBand2_4)
            return (CY_GET_REG8(pwmGenerator_DEADBAND_COUNT_PTR));
        #else

            /* Otherwise the data has to be masked and offset */
            #if(pwmGenerator_DEADBAND_COUNT_SHIFT)
                return ((uint8)(((uint8)(pwmGenerator_DEADBAND_COUNT & pwmGenerator_DEADBAND_COUNT_MASK)) >>
                                                                           pwmGenerator_DEADBAND_COUNT_SHIFT));
            #else
                return (pwmGenerator_DEADBAND_COUNT & pwmGenerator_DEADBAND_COUNT_MASK);
            #endif /* (pwmGenerator_DEADBAND_COUNT_SHIFT) */
        #endif /* (!pwmGenerator_DeadBand2_4) */
    }
#endif /* DeadBandUsed */

#if (pwmGenerator_UseStatus || pwmGenerator_UsingFixedFunction)


    /*******************************************************************************
    * Function Name: pwmGenerator_SetInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  This function configures the interrupts mask control of theinterrupt
    *  source status register.
    *
    * Parameters:
    *  uint8 interruptMode: Bit field containing the interrupt sources enabled
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void pwmGenerator_SetInterruptMode(uint8 interruptMode) 
    {
        CY_SET_REG8(pwmGenerator_STATUS_MASK_PTR, interruptMode);
    }


    /*******************************************************************************
    * Function Name: pwmGenerator_ReadStatusRegister
    ********************************************************************************
    *
    * Summary:
    *  This function returns the current state of the status register.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8 : Current status register value. The status register bits are:
    *  [7:6] : Unused(0)
    *  [5]   : Kill event output
    *  [4]   : FIFO not empty
    *  [3]   : FIFO full
    *  [2]   : Terminal count
    *  [1]   : Compare output 2
    *  [0]   : Compare output 1
    *
    *******************************************************************************/
    uint8 pwmGenerator_ReadStatusRegister(void) 
    {
        return (CY_GET_REG8(pwmGenerator_STATUS_PTR));
    }

#endif /* (pwmGenerator_UseStatus || pwmGenerator_UsingFixedFunction) */


#if (pwmGenerator_UseControl)


    /*******************************************************************************
    * Function Name: pwmGenerator_ReadControlRegister
    ********************************************************************************
    *
    * Summary:
    *  Returns the current state of the control register. This API is available
    *  only if the control register is not removed.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8 : Current control register value
    *
    *******************************************************************************/
    uint8 pwmGenerator_ReadControlRegister(void) 
    {
        uint8 result;

        result = CY_GET_REG8(pwmGenerator_CONTROL_PTR);
        return (result);
    }


    /*******************************************************************************
    * Function Name: pwmGenerator_WriteControlRegister
    ********************************************************************************
    *
    * Summary:
    *  Sets the bit field of the control register. This API is available only if
    *  the control register is not removed.
    *
    * Parameters:
    *  uint8 control: Control register bit field, The status register bits are:
    *  [7]   : PWM Enable
    *  [6]   : Reset
    *  [5:3] : Compare Mode2
    *  [2:0] : Compare Mode2
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void pwmGenerator_WriteControlRegister(uint8 control) 
    {
        CY_SET_REG8(pwmGenerator_CONTROL_PTR, control);
    }

#endif /* (pwmGenerator_UseControl) */


#if (!pwmGenerator_UsingFixedFunction)


    /*******************************************************************************
    * Function Name: pwmGenerator_ReadCapture
    ********************************************************************************
    *
    * Summary:
    *  Reads the capture value from the capture FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8/uint16: The current capture value
    *
    *******************************************************************************/
    uint8 pwmGenerator_ReadCapture(void) 
    {
        return (CY_GET_REG8(pwmGenerator_CAPTURE_LSB_PTR));
    }

#endif /* (!pwmGenerator_UsingFixedFunction) */


#if (pwmGenerator_UseOneCompareMode)


    /*******************************************************************************
    * Function Name: pwmGenerator_ReadCompare
    ********************************************************************************
    *
    * Summary:
    *  Reads the compare value for the compare output when the PWM Mode parameter is
    *  set to Dither mode, Center Aligned mode, or One Output mode.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8/uint16: Current compare value
    *
    *******************************************************************************/
    uint8 pwmGenerator_ReadCompare(void) 
    {
        #if(pwmGenerator_UsingFixedFunction)
            return ((uint8)CY_GET_REG16(pwmGenerator_COMPARE1_LSB_PTR));
        #else
            return (CY_GET_REG8(pwmGenerator_COMPARE1_LSB_PTR));
        #endif /* (pwmGenerator_UsingFixedFunction) */
    }

#else


    /*******************************************************************************
    * Function Name: pwmGenerator_ReadCompare1
    ********************************************************************************
    *
    * Summary:
    *  Reads the compare value for the compare1 output.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8/uint16: Current compare value.
    *
    *******************************************************************************/
    uint8 pwmGenerator_ReadCompare1(void) 
    {
        return (CY_GET_REG8(pwmGenerator_COMPARE1_LSB_PTR));
    }


    /*******************************************************************************
    * Function Name: pwmGenerator_ReadCompare2
    ********************************************************************************
    *
    * Summary:
    *  Reads the compare value for the compare2 output.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8/uint16: Current compare value.
    *
    *******************************************************************************/
    uint8 pwmGenerator_ReadCompare2(void) 
    {
        return (CY_GET_REG8(pwmGenerator_COMPARE2_LSB_PTR));
    }

#endif /* (pwmGenerator_UseOneCompareMode) */


/*******************************************************************************
* Function Name: pwmGenerator_ReadPeriod
********************************************************************************
*
* Summary:
*  Reads the period value used by the PWM hardware.
*
* Parameters:
*  None
*
* Return:
*  uint8/16: Period value
*
*******************************************************************************/
uint8 pwmGenerator_ReadPeriod(void) 
{
    #if(pwmGenerator_UsingFixedFunction)
        return ((uint8)CY_GET_REG16(pwmGenerator_PERIOD_LSB_PTR));
    #else
        return (CY_GET_REG8(pwmGenerator_PERIOD_LSB_PTR));
    #endif /* (pwmGenerator_UsingFixedFunction) */
}

#if ( pwmGenerator_KillModeMinTime)


    /*******************************************************************************
    * Function Name: pwmGenerator_WriteKillTime
    ********************************************************************************
    *
    * Summary:
    *  Writes the kill time value used by the hardware when the Kill Mode
    *  is set to Minimum Time.
    *
    * Parameters:
    *  uint8: Minimum Time kill counts
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void pwmGenerator_WriteKillTime(uint8 killtime) 
    {
        CY_SET_REG8(pwmGenerator_KILLMODEMINTIME_PTR, killtime);
    }


    /*******************************************************************************
    * Function Name: pwmGenerator_ReadKillTime
    ********************************************************************************
    *
    * Summary:
    *  Reads the kill time value used by the hardware when the Kill Mode is set
    *  to Minimum Time.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8: The current Minimum Time kill counts
    *
    *******************************************************************************/
    uint8 pwmGenerator_ReadKillTime(void) 
    {
        return (CY_GET_REG8(pwmGenerator_KILLMODEMINTIME_PTR));
    }

#endif /* ( pwmGenerator_KillModeMinTime) */

/* [] END OF FILE */
